unit MainFormU;

interface

uses
  System.Permissions,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.Maps;

type
  TMainForm = class(TForm)
    Button1: TButton;
    MapPanel: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FPermissionCoarseLocation: string;
    FPermissionFineLocation: string;
    FMapView: TMapView;
    procedure LocationPermissionRequestResult(Sender: TObject; const APermissions: TArray<string>; const AGrantResults: TArray<TPermissionStatus>);
    procedure SetupMap(ALocationPermissionGranted: Boolean);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$INLINE AUTO}

uses
{$IFDEF ANDROID}
  Androidapi.Jni.Os,
  Androidapi.Jni.Widget,
  Androidapi.Helpers,
  FMX.Helpers.Android,
{$ENDIF}
  FMX.DialogService;

{$R *.fmx}

{$IFDEF ANDROID}
procedure Toast(const Msg: string; Duration: Integer);
begin
  CallInUiThread(
    procedure
    begin
      TJToast.JavaClass.makeText(
        TAndroidHelper.Context,
        StrToJCharSequence(msg),
        Duration).show
    end);
end;
{$ENDIF}

procedure TMainForm.FormCreate(Sender: TObject);
begin
{$IFDEF ANDROID}
  // Note we can alternatively use literal strings:
  //   'android.permission.ACCESS_COARSE_LOCATION'
  //   'android.permission.ACCESS_FINE_LOCATION'
  // according to the documentation:
  //   https://developer.android.com/reference/android/Manifest.permission#ACCESS_COARSE_LOCATION
  //   https://developer.android.com/reference/android/Manifest.permission#ACCESS_FINE_LOCATION
  FPermissionCoarseLocation := JStringToString(TJManifest_permission.JavaClass.ACCESS_COARSE_LOCATION);
  FPermissionFineLocation := JStringToString(TJManifest_permission.JavaClass.ACCESS_FINE_LOCATION);
{$ENDIF}
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  PermissionsService.RequestPermissions([FPermissionCoarseLocation, FPermissionFineLocation], LocationPermissionRequestResult);
end;

procedure TMainForm.LocationPermissionRequestResult(Sender: TObject; const APermissions: TArray<string>; const AGrantResults: TArray<TPermissionStatus>);
begin
  // 2 permissions involved: ACCESS_COARSE_LOCATION and ACCESS_FINE_LOCATION
  var LocationPermissionGranted := (Length(AGrantResults) = 2) and (AGrantResults[0] = TPermissionStatus.Granted) and (AGrantResults[1] = TPermissionStatus.Granted);
  if LocationPermissionGranted then
    Toast('User granted permission', TJToast.JavaClass.LENGTH_SHORT)
  else
    Toast('User denied permission', TJToast.JavaClass.LENGTH_SHORT);

  SetupMap(LocationPermissionGranted);
end;

procedure TMainForm.SetupMap(ALocationPermissionGranted: Boolean);
begin
  // Remember to set up the API key in the Version Info page of the Projct options dialog, as per:
  // http://docwiki.embarcadero.com/RADStudio/en/Configuring_Android_Applications_to_Use_Google_Maps#Configuring_Specific_Project_Options
  if FMapView = nil then
  begin
    FMapView := TMapView.Create(Self);
    FMapView.Align := TAlignLayout.Client;
    FMapView.LayerOptions := [TMapLayerOption.PointsOfInterest, TMapLayerOption.Buildings, TMapLayerOption.Traffic];
    FMapView.ControlOptions := [TMapControlOption.Zoom, TMapControlOption.Compass];
    FMapView.Parent := MapPanel;
  end;
  if ALocationPermissionGranted then
  begin
    // These options only get respected/actioned if the app has one or both access location permissions
    FMapView.LayerOptions := FMapView.LayerOptions + [TMapLayerOption.UserLocation];
    FMapView.ControlOptions := FMapView.ControlOptions + [TMapControlOption.MyLocation];
  end;
end;

end.
